# -*- coding: utf-8 -*-
"""
Une correction du TP sur l'approximation numérique

"""
import matplotlib.pyplot as plt
import time
    
def dichotomie(fonction,u0,v0,epsilon):
    # fonction(x)=0 admet une unique solution alpha avec
    # u0<alpha<v0. epsilon est un nombre >0.
    # dichotomie renvoie une approximation de alpha avec une précision epsilon
    
    u=u0
    v=v0
    while v-u>epsilon:
        milieu=(v+u)/2
        if fonction(milieu)*fonction(u)<0:
            v=milieu
        else:
            u=milieu  
    return((u+v)/2)
    
    
    
def newton(fonction,derive,u0,n):
    # fonction(x)=0 a une solution alpha, derive est la fonction dérivée
    # u0 : premiere approximation
    # n : nombre d'itérations
    # newton renvoie la valeur obtenue après n itérations de la méthode de Newton
    
    u=u0
    i=1
    while i<n+1:
        u=u-fonction(u)/derive(u)
        i=i+1
    return(u)
    

# Les fonctions f et df nous servent pour notre exemple x^3+3x+1=0

def g(x):
    return(pow(x,3)+3*x+1)
    

def dg(x):
    return(3*pow(x,2)+3)
  
    
    
def mon_newton(f,df,u0,epsilon):
    # f est une fonction, df sa dérivée. On veut approcher la solution
    # de f(x)=0 à l'aide de la méthode de Newton.
    # uo et la première approximation, on la suppose assez proche de
    # la solution exacte pour assurer la convergence de la suite.
    # espilon>0 est la précision souhaitée
    terme=u0
    test=True
    while test:
        nouveauterme=-f(terme)/df(terme)+terme
        test=abs(terme-nouveauterme)>epsilon
        terme=nouveauterme
    return(terme)    

    
def dicho_recursif(f,u0,v0,epsilon):
    milieu =(u0+v0)/2
    if v0-u0<=2*epsilon :
        return(milieu)
    elif f(milieu)>0:
        return(dicho_recursif(f,u0,milieu,epsilon))
    else:
        return(dicho_recursif(f,milieu,v0,epsilon))
        

###################################################################
#                                                                 #
#                  COMPARAISON DES FONCTIONS                      #
#                                                                 #
###################################################################        
        
        
# on compare le temps d'exécution des fonctions pour une précision donnée
# on aura donc, en ordonnée, le temps qui va être mesuré grace à la fonction
# time du module time.
# En abscisse, le graphique serait difficilement lisible si on mettait 
# la précision. Comprenez-vous pourquoi? Essayez !
# Si la précision est 10^(-n), on va donc prendre n en abscisse (soit -log(epsilon)) 



def compareDichoNewton():
    # affiche les courbes du temps d'execution pour obtenir une valeur 
    # approchée par les différentes fonctions en fonction de la qualité 
    # de la précision demandée
    
    precision=[10**(-n) for n in range(1,11)]
    tempsDicho,tempsDichoRecursif,tempsNewton=[],[],[]
    
    for epsilon in precision:
        # pour chaque valeur de epsilon, on mesure le temps d'execution
        tempsDebut=time.time()
        dichotomie(g,-1,0,epsilon)
        tempsEcoule=time.time()-tempsDebut
        tempsDicho.append(tempsEcoule)
        
        tempsDebut=time.time()
        dicho_recursif(g,-1,0,epsilon)
        tempsEcoule=time.time()-tempsDebut
        tempsDichoRecursif.append(tempsEcoule)
        
        tempsDebut=time.time()
        mon_newton(g,dg,0,epsilon)
        tempsEcoule=time.time()-tempsDebut
        tempsNewton.append(tempsEcoule)
    
    logPrecision=range(1,11)
    
    plt.plot(logPrecision,tempsDicho,'-r', label="dichotomie")
    plt.plot(logPrecision,tempsDichoRecursif,'-b', label="dichotomie récursif")
    plt.plot(logPrecision,tempsNewton,'-g', label="Newton")
    plt.xlabel("qualité de la précision")
    plt.ylabel("temps de calcul")
    plt.legend()
    


def compareNewton():
    import scipy.optimize as opti
    precision=[10**(-n) for n in range(1,11)]
    tempsMonNewton,tempsNewtonScipy,tempsNewtonScipySansDeriv=[],[],[]
    
    for epsilon in precision:
        # pour chaque valeur de epsilon, on mesure le temps d'execution
        
        tempsDebut=time.time()
        mon_newton(g,dg,0,epsilon)
        tempsEcoule=time.time()-tempsDebut
        tempsMonNewton.append(tempsEcoule)
    
        
        tempsDebut=time.time()
        opti.newton(g,0,dg,tol=epsilon)
        tempsEcoule=time.time()-tempsDebut
        tempsNewtonScipy.append(tempsEcoule)
    
        # la fonction newton de scipy prévoit le cas où on ne dispose pas de
        # dérivée, c'est alors la sécante qui approche la tangente
        tempsDebut=time.time()
        opti.newton(g,0,tol=epsilon)
        tempsEcoule=time.time()-tempsDebut
        tempsNewtonScipySansDeriv.append(tempsEcoule)
    
    logPrecision=range(1,11)
    
    plt.plot(logPrecision,tempsMonNewton,'-r', label="notre implémentation")
    plt.plot(logPrecision,tempsNewtonScipy,'-b', label="scipy avec dérivée")
    plt.plot(logPrecision,tempsNewtonScipySansDeriv,'-g', label="scipy sans dérivée")
    plt.xlabel("qualité de la précision")
    plt.ylabel("temps de calcul")
    plt.legend()